/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.ow2.jonas.ws.jaxws.http.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

public class JOnASServletRequestWrapper {

    private int contentLength;
    private String contentType;
    private InputStream in;
    private String method;
    private Map parameters;
    private URI uri;
    private URL url;
    private Map headers;
    private Map attributes;
    private String remoteAddress;
    private String qureyString;

    private HttpServletRequest request;


    public JOnASServletRequestWrapper(final HttpServletRequest request) {
        this.request = request;
    }

    //this constructor is for test
    public JOnASServletRequestWrapper(final int contentLength, final String contentType, final InputStream in,
            final String method,final Map parameters, final URI uri, final Map headers, final URL url,
            final String remoteAddress,final String queryString) {
        this.contentType = contentType;
        this.contentLength = contentLength;
        this.in = in;
        this.method = method;
        this.parameters = parameters;
        this.uri = uri;
        this.url = url;
        this.headers = headers;
        this.attributes = new HashMap();
        this.remoteAddress = remoteAddress;
        this.qureyString = queryString;
        this.request = null;
    }

    public HttpServletRequest getRequest() {
        return request;
    }

    public int getContentLength() {
        if(request != null) {
            return request.getContentLength();
        } else {
            return contentLength;
        }
    }

    public String getContentType() {
        if(request != null) {
            return request.getContentType();
        } else {
            return contentType;
        }
    }

    public String getHeader(final String name) {
        if(request != null) {
            return request.getHeader(name);
        } else {
            return (String) headers.get(name);
        }
    }

    public InputStream getInputStream() throws IOException{
        if(request != null) {
            return request.getInputStream();
        } else {
            return in;
        }
    }

    public String getMethod() {
        if(request != null) {
            return request.getMethod();
        } else {
            return method;
        }
    }

    public String getParameter(final String name) {
        if(request != null) {
            return request.getParameter(name);
        } else {
            return (String) parameters.get(name);
        }
    }

    public Map getParameters() {
        if(request != null) {
            return request.getParameterMap();
        } else {
            return parameters;
        }
    }

    public String getRequestURI() {
        if(request != null) {
            return request.getRequestURI();
        } else {
            return uri.toString();
        }
    }

    public String getRemoteAddr() {
        if(request != null) {
            return request.getRemoteAddr();
        } else {
            return remoteAddress;
        }
    }

    public StringBuffer getRequestURL() {
        // TODO Auto-generated method stub
        if(request != null) {
            return request.getRequestURL();
        } else {
            return request.getRequestURL();
        }
    }

    public Object getAttribute(final String name) {
        return attributes.get(name);
    }

    public void setAttribute(final String name, final Object value) {
        attributes.put(name, value);
    }

    public String getContextPath() {
        return "/";
    }

    public String getQueryString() {
        if(request != null) {
            return request.getQueryString();
        } else {
            return qureyString;
        }
    }



}
