/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.ow2.jonas.ws.jaxws.http.servlet;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

public class JOnASServletResponseWrapper {
    private String contentType;
    private String host;
    private OutputStream out;
    private String method;
    private Map parameters;
    private String path;
    private URL uri;
    private int port;
    private Map headers;
    private int statusCode;
    private String statusMessage;

    private HttpServletResponse response;

    public JOnASServletResponseWrapper(final HttpServletResponse response) {
        this.response = response;
    }

    // this constructor is for test
    public JOnASServletResponseWrapper(final String contentType,final String host,final String path, final URL uri,
            final int port ,final OutputStream out) {
        this.contentType = contentType;
        this.host = host;
        this.parameters = new HashMap();
        this.path = path;
        this.uri = uri;
        this.port = port;
        this.headers = new HashMap();
        this.out = out;
        this.response = null;
    }

    public HttpServletResponse getResponse() {
        return response;
    }

    public void setContentLength(final int length) {
        if(response != null) {
            response.setContentLength(length);
        }
    }
    public void setHeader(final String name,final String value) {
        if(response != null) {
            response.setHeader(name, value);
        }
    }
    public String getHost() {
        return host;
    }
    public OutputStream getOutputStream() throws IOException {
        if(response != null) {
            return response.getOutputStream();
        } else {
            return out;
        }
    }
    public String getMethod() {
        return method;
    }
    public String getParameter(final String name) {
        return (String) parameters.get(name);
    }
    public Map getParameters() {
        return parameters;
    }

    public String getPath() {
        return path;
    }

    public int getPort() {
        return port;
    }
    public URL getURI() {
        return uri;
    }

    public String getContentType() {
        if(response != null) {
            return response.getContentType();
        } else {
            return contentType;
        }
    }
    public void setStatusCode(final int code) {
        if(response != null) {
            response.setStatus(code);
        } else {
            statusCode =code;
        }
    }
    public int getStatusCode() {
        return statusCode;
    }
    public void setStatusMessage(final String responseString) {
        statusMessage = responseString;
    }

    public void setStatus(final int code, final String responseString) {
        if(response != null) {
            response.setStatus(code, responseString);
        } else {
            statusMessage = responseString;
            statusCode =code;
        }
    }
}
